// Copyright 2004, FreeHEP.
package hep.wired.heprep.edit;

import hep.wired.edit.GraphicsPanelEdit;
import hep.wired.edit.WiredEdit;
import hep.wired.heprep.graphicspanel.HepRepPanel;
import hep.wired.heprep.services.Projection;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.util.logging.Logger;

import javax.swing.undo.UndoableEdit;

/**
 * Set a projection for the plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: SetProjection.java 8598 2006-08-14 20:38:02Z duns $
 */
public class SetProjection extends WiredEdit implements GraphicsPanelEdit {

    private static Logger logger = Logger.getLogger(SetProjection.class
            .getPackage().getName());

    private Projection oldProjection;

    private Projection projection;

    /**
     * Creates a projection edit with given projection.
     */
    public SetProjection(Projection projection) {
        super();

        this.projection = projection;
    }

    public WiredEdit copy(RecordPlot plot) {
        WiredEdit copy = new SetProjection(projection.copy());
        copy.setRecordPlot(plot);

        return copy;
    }

    public String getPresentationName() {
        return "SetProjection: " + projection.getName();
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        return p instanceof HepRepPanel;
    }

    public boolean addEdit(UndoableEdit edit) {
        if (edit instanceof SetProjection) {
            return edit.equals(this);
        }
        return super.addEdit(edit);
    }

    protected void redoEdit() {
        RecordPlot plot = getRecordPlot();
        HepRepPanel panel = (HepRepPanel) plot.getGraphicsPanel();
        oldProjection = panel.getProjection();
        panel.setProjection(projection);
        plot.repaint();
        logger.finer(toString());
    }

    protected void undoEdit() {
        RecordPlot plot = getRecordPlot();
        HepRepPanel panel = (HepRepPanel) plot.getGraphicsPanel();
        projection = panel.getProjection();
        panel.setProjection(oldProjection);
        plot.repaint();
        logger.finer(toString());
    }
}
