// Copyright 2005-2011, FreeHEP.
package hep.wired.heprep.graphicspanel;

import hep.graphics.heprep.HepRep;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.services.InteractionHandler;

import javax.swing.Icon;
import javax.swing.JPanel;

/**
 * Base class for control panels used by WIRED.
 *
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: ControlPanel.java 13833 2011-05-10 23:26:00Z onoprien $
 */
abstract public class ControlPanel extends JPanel {

// -- Private parts : ----------------------------------------------------------

  private Icon _icon;
  private InteractionHandler _interactionHandler;
  private boolean _isSelected;

  /** Default interaction handler currently associated with this panel. */
  protected InteractionHandler _defaultInteractionHandler;


// -- Construction : -----------------------------------------------------------

  public ControlPanel(String name, String toolTip, Icon icon, InteractionHandler defaultInteractionHandler) {
    setName(name);
    _icon = icon;
    setToolTipText(toolTip);
    _interactionHandler = defaultInteractionHandler;
    _defaultInteractionHandler = defaultInteractionHandler;
    _isSelected = false;
  }


// -- Setters : ----------------------------------------------------------------

  /** Sets icon associated with this panel. */
  public void setIcon(Icon icon) {
    _icon = icon;
  }

  /**
   * Sets interaction handler associated with this panel.
   * Implementation provided by this class simply saves a reference to the handler,
   * so it can be retrieved later by a call to <tt>getInteractionHandler()</tt>.
   * Subclasses may override to include additional initialization.
   */
  public void setInteractionHandler(InteractionHandler interactionHandler) {
    _interactionHandler = interactionHandler;
  }


// -- Getters : ----------------------------------------------------------------

  /** Returns icon associated with this panel. */
  public Icon getIcon() {
    return _icon;
  }

  /** Returns <tt>true</tt> if this panel is currently selected. */
  public boolean isSelected() {
    return _isSelected;
  }

  /**
   * Returns interaction handler currently associated with this panel.
   * Implementation provided by this class returns the handler previously set with
   * a call to <tt>setInteractionHandler(...)</tt>.
   */
  public InteractionHandler getInteractionHandler() {
    return _interactionHandler;
  }

  /**
   * Returns default interaction handler currently associated with this panel.
   * Implementation provided by this class returns the handler supplied to the constructor.
   */
  public InteractionHandler getDefaultInteractionHandler() {
    return _defaultInteractionHandler;
  }


// -- Responding to plugin-level events : --------------------------------------

  /**
   * Called when this panel is selected or deselected.
   * Implementation provided by this class sets current interaction handler on this panel
   * when the panel is selected.
   */
  public void setSelected(boolean selected) {
//    System.out.println("ControlPanel "+ this.getName() + "setSelected("+ selected +")");
    _isSelected = selected;
    if (selected) {
      InteractionHandler ih = WiredPlugin.getPlugin().getInteractionHandler();
      setInteractionHandler(ih == null ? _defaultInteractionHandler : ih);
    }
  }

  /** 
   * Called when a new record is loaded into WIRED plugin.
   * This class provides empty implementation.
   */
  public void setRecord(HepRep heprep) {}
}
