package hep.wired.heprep.interaction;

import java.util.*;

import hep.wired.heprep.util.Keys;
import hep.wired.util.SortableListTableModel;
import org.freehep.application.Application;
import org.freehep.application.PropertyUtilities;

/**
 * TableModel for HepRep Attribute Categories.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: CategoryTableModel.java 13822 2011-03-12 22:19:14Z onoprien $
 */
public class CategoryTableModel extends SortableListTableModel {

// -- Construction : -----------------------------------------------------------

  public CategoryTableModel(List<Flag> categories) {
    super(new String[]{"Category", "Shown"}, new Class[]{String.class, Boolean.class}, categories);
  }

// -- Getters : ----------------------------------------------------------------

  public Set<String> getShownSet() {
    HashSet<String> out = new HashSet();
    for (Object o : keys) {
      Flag f = (Flag)o;
      if (f.isSet()) out.add(f.getName());
    }
    return out;
  }

// -- Implementing TableModel : ------------------------------------------------

  public boolean isCellEditable(int row, int col) {
    return (col != Flag.NAME);
  }

  public Object getValueAt(int row, int col) {
    Flag category = (Flag) keys.get(row);
    switch (col) {
      case Flag.NAME:
        return category.getName();
      case Flag.FLAG:
        return (Boolean) category.isSet();
      default:
        return "";
    }
  }

  public void setValueAt(Object value, int row, int col) {
    switch (col) {
      case Flag.NAME:
        return;
      case Flag.FLAG:
        Flag category = (Flag) keys.get(row);
        category.set((Boolean) value);
        changed();
        return;
      default:
        return;
    }
  }

  protected int compareKeys(Object o1, Object o2, int col, boolean ascending) {
    Flag c1 = (Flag) o1;
    Flag c2 = (Flag) o2;
    switch (col) {
      default:
      case Flag.NAME:
        return (ascending ? 1 : -1) * (c1.getName().compareTo(c2.getName()));
      case Flag.FLAG:
        return (ascending ? 1 : -1) * (c1.isSet() ? (c2.isSet() ? 0 : 1) : c2.isSet() ? -1 : 0);
    }
  }

  // Called when the user changes the table data (via GUI)
  protected void changed() {
  }

}
