// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.wired.heprep.image.WiredImage;
import hep.wired.interaction.AbstractInteractionHandler;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.GeneralPath;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.Icon;

import org.freehep.application.Application;

/**
 * Click mouse to set a location.
 * 
 * @author Mark Donszelmann
 * @version $Id: ClickToSetLocation.java 8598 2006-08-14 20:38:02Z duns $
 */
public class ClickToSetLocation extends AbstractInteractionHandler {

    // FIXME not used yet
    protected boolean showHairLines;

    private int mouseX = -1;

    private int mouseY = -1;

    private int x = -1;

    private int y = -1;

    private boolean xIsFixed, yIsFixed;

    private List/* <LocationListener> */listeners = new ArrayList();

    /**
     * Create a click to set location handler.
     */
    public ClickToSetLocation() {
        this(true);
    }

    /**
     * Create a click to set location handler.
     */
    public ClickToSetLocation(boolean showHairLines) {
        super("Click to Set a Location");
        this.showHairLines = showHairLines;
        xIsFixed = false;
        yIsFixed = false;
    }

    /**
     * Add a location listener, to be informed when the location changes.
     */
    public void addLocationListener(LocationListener listener) {
        listeners.add(listener);
    }

    /**
     * Remove a location listener.
     */
    public void removeLocationListener(LocationListener listener) {
        listeners.remove(listener);
    }

    public Icon getIcon(int size) {
        return WiredImage.getIcon("Translate3D%w", size);
    }

    public String getDescription() {
        return "Click to select a point in two orthogonal planes.";
    }

    public boolean isSupportedBy(GraphicsPanel panel) {
        return false;
        // return (new Translate().isSupportedBy(panel)) &&
        // panel.supports(Translateable3D.class);
    }

    public void changeCursor(RecordPlot plot, InputEvent event) {
        plot.setCursor(WiredImage.getBestCursor("TranslateCursor3D%w", 32, 32));
    }

    public void reset(RecordPlot plot, InputEvent event) {
        x = mouseX;
        y = mouseY;
        xIsFixed = false;
        yIsFixed = false;
        defineShape(plot);
        informListeners(plot);
        Application.getApplication().setStatusMessage(getDescription());
    }

    public void mouseMoved(RecordPlot plot, MouseEvent event) {
        modifyPoint(event);
        defineShape(plot);
        informListeners(plot);
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        plot.requestFocusInWindow();
        modifyPoint(event);
        defineShape(plot);
        informListeners(plot);
    }

    public void mouseExited(RecordPlot plot, MouseEvent event) {
        modifyPoint(event);
        defineShape(plot);
        informListeners(plot);
    }

    public void mouseButton1Clicked(RecordPlot plot, MouseEvent event) {
        modifyPoint(event);
        xIsFixed = true;
        yIsFixed = true;
        defineShape(plot);
        informListeners(plot);
    }

    /**
     * Sets (and fixes) the location of this handler.
     */
    public void setLocation(RecordPlot plot, int x, int y, boolean xIsFixed,
            boolean yIsFixed) {
        this.x = x;
        this.y = y;
        this.xIsFixed = xIsFixed;
        this.yIsFixed = yIsFixed;
        defineShape(plot);
        // plot.repaint();
    }

    private void informListeners(RecordPlot plot) {
        for (Iterator i = listeners.iterator(); i.hasNext();) {
            ((LocationListener) i.next()).setLocation(plot, x, y, xIsFixed,
                    yIsFixed);
        }
    }

    private void modifyPoint(MouseEvent event) {
        int id = event.getID();
        x = (xIsFixed) ? x : (id != MouseEvent.MOUSE_EXITED) ? event.getX()
                : -1;
        y = (yIsFixed) ? y : (id != MouseEvent.MOUSE_EXITED) ? event.getY()
                : -1;
        mouseX = (id != MouseEvent.MOUSE_EXITED) ? event.getX() : -1;
        mouseY = (id != MouseEvent.MOUSE_EXITED) ? event.getY() : -1;
    }

    private void defineShape(RecordPlot plot) {
        float w = plot.getWidth();
        float h = plot.getHeight();
        GeneralPath p = new GeneralPath();

        // horizontal
        if (y >= 0) {
            p.moveTo(0, y);
            p.lineTo(w, y);
        }

        // vertical
        if (x >= 0) {
            p.moveTo(x, 0);
            p.lineTo(x, h);
        }
        plot.drawShape(p);
    }

    public String toString() {
        return "Click to set a location";
    }
}
