// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.wired.heprep.edit.SetVariable;
import hep.wired.image.WiredBaseImage;
import hep.wired.interaction.DefaultInteractionHandler;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;

import javax.swing.Icon;

import org.freehep.application.Application;

/**
 * Drag/click mouse to scale plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: DragToChangeClock.java 8598 2006-08-14 20:38:02Z duns $
 */
public class DragToChangeClock extends DefaultInteractionHandler {

    private static final String phi0String = "phi0";

    private double phi0;

    private static final String dString = "d";

    /**
     * Create a drag handler to generate scale edits.
     */
    public DragToChangeClock() {
        super("Drag to Change Opening Angle of Clock");
    }

    public String getDescription() {
        return "Drag to open clock angle.";
    }
    
    public boolean isSupportedBy(GraphicsPanel panel) {
        return new SetVariable(phi0String).isSupportedBy(panel)
                && new SetVariable(dString).isSupportedBy(panel);
    }

    public Icon getIcon(int size) {
        // FIXME, WIRED-318, other icon...
        return WiredBaseImage.getIcon("Scale%w", size);
    }

    public void changeCursor(RecordPlot plot, InputEvent event) {
        // FIXME, WIRED-318, other icon.
        plot.setCursor(WiredBaseImage.getBestCursor("ScaleCursor%w", 32, 32));
    }

    public void reset(RecordPlot plot, InputEvent event) {
        plot.getGraphicsPanel().setFastMode(false);
        plot.drawShape(null);
        plot.repaint();
        Application.getApplication().setStatusMessage(
                getDescription());
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        plot.requestFocusInWindow();
        changeCursor(plot, event);
    }

    public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
        phi0 = Math.atan2(
                event.getComponent().getHeight() / 2.0 - event.getY(), event
                        .getX()
                        - event.getComponent().getWidth() / 2.0);
        changeCursor(plot, event);
        plot.beginUpdate();
        plot.getGraphicsPanel().setFastMode(true);
        plot.postEdit(new SetVariable(phi0String, phi0));
    }

    public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
        double d = Math.abs(Math.atan2(event.getComponent().getHeight() / 2.0
                - event.getY(), event.getX() - event.getComponent().getWidth()
                / 2.0)
                - phi0);

        // D incorrect for dragging over the -PI or +PI boundary...
        plot.postEdit(new SetVariable(dString, d / Math.PI));
        defineShape(plot, event);
    }

    public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
        plot.endUpdate();
        reset(plot, event);
    }

    public void mouseButton1Released(RecordPlot plot, MouseEvent event) {
        changeCursor(plot, event);
    }

    private void defineShape(RecordPlot plot, MouseEvent event) {
        // double x = event.getComponent().getWidth()/2.0;
        // double y = event.getComponent().getHeight()/2.0;
        // double d = Point2D.distance(event.getX(), event.getY(), x, y);
        // plot.drawShape(new Ellipse2D.Double(x - d, y - d, d*2, d*2));
    }

    public String toString() {
        return "Drag to change Clock";
    }
}
