// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.interaction;

import hep.wired.heprep.edit.MultiplyVariable;
import hep.wired.heprep.edit.SetVariable;
import hep.wired.image.WiredBaseImage;
import hep.wired.interaction.DefaultInteractionHandler;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import javax.swing.Icon;

import org.freehep.application.Application;

/**
 * Drag/click mouse to scale plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: DragToChangeFishEye.java 8598 2006-08-14 20:38:02Z duns $
 */
public class DragToChangeFishEye extends DefaultInteractionHandler {

    private static final String variable = "alpha";

    private double dpsq;

    /**
     * Create a drag handler to generate scale edits.
     */
    public DragToChangeFishEye() {
        super("Drag to Change Deformation of Fish-Eye");
    }

    public String getDescription() {
        return "Drag inward or outward to change blow-up factor.";
    }
    
    public boolean isSupportedBy(GraphicsPanel panel) {
        return new SetVariable(variable).isSupportedBy(panel);
    }

    public Icon getIcon(int size) {
        // FIXME, WIRED-378, other icon.
        return WiredBaseImage.getIcon("Scale%w", size);
    }

    public void changeCursor(RecordPlot plot, InputEvent event) {
        // FIXME, WIRED-378, other icon.
        plot.setCursor(WiredBaseImage.getBestCursor("ScaleCursor%w", 32, 32));
    }

    public void reset(RecordPlot plot, InputEvent event) {
        plot.getGraphicsPanel().setFastMode(false);
        plot.drawShape(null);
        plot.repaint();
        Application.getApplication().setStatusMessage(
                getDescription());
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        plot.requestFocusInWindow();
        changeCursor(plot, event);
    }

    public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
        dpsq = Point2D.distanceSq(event.getComponent().getWidth() / 2.0, event
                .getComponent().getHeight() / 2.0, event.getX(), event.getY());
        changeCursor(plot, event);
        plot.beginUpdate();
        plot.getGraphicsPanel().setFastMode(true);
    }

    public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
        double ddsq = Point2D.distanceSq(event.getComponent().getWidth() / 2.0,
                event.getComponent().getHeight() / 2.0, event.getX(), event
                        .getY());
        double s = Math.sqrt(ddsq / dpsq);
        // double s = Math.sqrt(ddsq)/2000;

        plot.postEdit(new MultiplyVariable(variable, s));
        dpsq = ddsq;
        defineShape(plot, event);
    }

    public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
        plot.endUpdate();
        reset(plot, event);
    }

    public void mouseButton1Released(RecordPlot plot, MouseEvent event) {
        changeCursor(plot, event);
    }

    private void defineShape(RecordPlot plot, MouseEvent event) {
        double x = event.getComponent().getWidth() / 2.0;
        double y = event.getComponent().getHeight() / 2.0;
        double d = Point2D.distance(event.getX(), event.getY(), x, y);
        plot.drawShape(new Ellipse2D.Double(x - d, y - d, d * 2, d * 2));
    }

    public String toString() {
        return "Drag to change Fish Eye";
    }
}
