package hep.wired.heprep.interaction;

import hep.graphics.heprep.HepRep;
import hep.wired.util.tree.WiredTreeModel.MarkEvent;
import hep.wired.util.tree.WiredTreeNode;
import java.util.*;

import hep.graphics.heprep.HepRepType;
import hep.wired.cut.Cut;
import hep.wired.cut.Cut.Result;
import hep.wired.heprep.tree.HepRepTreeModel;
import hep.wired.util.tree.WiredTreeModel;
import org.freehep.util.Value;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class HepRepCut extends Cut implements WiredTreeModel.MarkListener {

// -- Private parts : ----------------------------------------------------------

  private Set<HepRepType> _types;
  private HepRepTreeModel _typeTree;

// -- Construction and initialization : ----------------------------------------

  public HepRepCut() {
    this(_defaultName);
  }

  public HepRepCut(String name) {
    super(name);
  }

// -- Getters : ----------------------------------------------------------------

  public Set<HepRepType> getTypes() {
    return _types;
  }

  public HepRepTreeModel getTypeTree() {
    return _typeTree;
  }

// -- Handling HepRepTypes : ---------------------------------------------------

  public void setTypesTree(HepRepTreeModel treeModel) {
    if (_typeTree != null) _typeTree.removeMarkListener(this);
    _typeTree = treeModel;
    if (treeModel != null) {
      _typeTree.addMarkListener(this);
      markChanged(null);
    } else {
      _types = null;
    }
  }

  public void setRecord(HepRep heprep) {
//    if (_typesTree != null) {
//      HepRepTreeModel tt = _typesTree.clone(heprep);
//      setTypesTree(this);
//    }
  }

  public void markChanged(MarkEvent e) {
    _types = _typeTree.getMarkedObjects();
    WiredTreeModel.TreeVisitor visitor = new WiredTreeModel.TreeVisitor() {
      public boolean visit(WiredTreeNode node) {
        if ((node.isInUnmarkedBranch() || !node.isMarkedNode()) && !_types.contains(node.getPayload())) {
          throw new RuntimeException();
        }
        return true;
      }
    };
    try {
      visitor.visitNodes(_typeTree);
      _types = null;
    } catch (RuntimeException x) {}
    fireStateChanged(_defaultEvent);
  }

// -- Apply cut : --------------------------------------------------------------

  /**
   * Tells the client whether this cut is satisfied by the supplied data.
   * The implementation provided by this class expects <tt>moreData</tt> parameter
   * to be an instance of <tt>HepRepType</tt>, and returns IGNORE if this cut is
   * not applicable. Otherwise, the call is forwarded to
   * {@link Cut#pass(java.util.Map, boolean)}.
   *
   * @param data  Map of attribute names to values.
   * @param moreData  <tt>HepRepType</tt> of the object being filtered.
   * @param invert  True if this cut should be inverted.
   * @return  PASS if the data satisfies this cut, FAIL if the data fails the cut, and IGNORE if the cut is inapplicable.
   */
  public Result pass(Map<String,Value> data, Object moreData, boolean invert) {
    if (_types == null || _types.contains(moreData)) {
      return pass(data, invert);
    } else {
      return Result.IGNORE;
    }
  }

// -----------------------------------------------------------------------------
}
