package hep.wired.heprep.interaction;

import java.util.*;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.util.SortableListTableModel;

/**
 * Model for the Table of picked/selected HepRep instances in {@link HepRepInfoPanel}.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: InstanceTableModel.java 13822 2011-03-12 22:19:14Z onoprien $
 */
public class InstanceTableModel extends SortableListTableModel {

// -- Private parts : ----------------------------------------------------------

  private static final int TYPENAME = 0;
  private static final int POINTS = 1;
  private static final int INSTANCES = 2;
  private static final int PICKPARENT = 3;

// -- Construction : -----------------------------------------------------------

  public InstanceTableModel() {
    this(Collections.<HepRepInstance>emptyList());
  }

  public InstanceTableModel(List<HepRepInstance> values) {
    super(new String[]{"Type", "Points", "Children", "PickParent"}, new Class[]{
              String.class, Integer.class, Integer.class, Boolean.class}, values);
  }

// -- Getters : ----------------------------------------------------------------

  public HepRepInstance get(int i) {return (HepRepInstance) keys.get(i);}

// -- Implementing TableModel : ------------------------------------------------

  public boolean isCellEditable(int row, int col) {
    return false;
  }

  public Object getValueAt(int row, int col) {
    HepRepInstance instance = (HepRepInstance) keys.get(row);
    switch (col) {
      case TYPENAME:
        return instance.getType().getName();
      case POINTS:
        return new Integer(instance.getPoints().size());
      case INSTANCES:
        return new Integer(instance.getInstances().size());
      case PICKPARENT:
        return (Boolean) instance.getAttValue("PickParent").getBoolean();
      default:
        return "";
    }
  }

  protected int compareKeys(Object o1, Object o2, int col, boolean ascending) {
    HepRepInstance i1 = (HepRepInstance) o1;
    HepRepInstance i2 = (HepRepInstance) o2;
    switch (col) {
      default:
      case TYPENAME:
        return (ascending ? 1 : -1)
                * (i1.getType().getName().compareTo(i2.getType().getName()));
      case POINTS:
        int n1 = i1.getPoints().size();
        int n2 = i2.getPoints().size();
        return (ascending ? 1 : -1) * (n1 < n2 ? -1 : n1 == n2 ? 0 : 1);
      case INSTANCES:
        int s1 = i1.getInstances().size();
        int s2 = i2.getInstances().size();
        return (ascending ? 1 : -1) * (s1 < s2 ? -1 : s1 == s2 ? 0 : 1);
      case PICKPARENT:
        boolean b1 = i1.getAttValue("PickParent").getBoolean();
        boolean b2 = i2.getAttValue("PickParent").getBoolean();
        return (ascending ? 1 : -1) * (b1 ? (b2 ? 0 : 1) : b2 ? -1 : 0);
    }
  }
}
