package hep.wired.heprep.interaction;

import hep.wired.heprep.graphicspanel.ControlPanel;
import hep.wired.heprep.plugin.WiredPlugin;
import hep.wired.heprep.tree.TreePanel;
import hep.wired.image.WiredBaseImage;
import hep.wired.plugin.WiredPage;
import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import org.freehep.application.mdi.PageContext;
import org.freehep.swing.layout.TableLayout;

/**
 * Control panel that contains {@link TreePanel} and buttons for choosing view control interaction handlers.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: InteractionPanel.java 13837 2011-05-12 23:12:27Z onoprien $
 */
public class InteractionPanel extends ControlPanel {
  
// -- Private parts : ----------------------------------------------------------

  private JPanel _interaction;
  private TreePanel _treePanel;

  private ButtonGroup _group;
  private Map<String, JToggleButton> _buttonsByName;
  private Map<InteractionHandler, JToggleButton> _buttonsByHandler;
  private JToggleButton _deselect;

// -- Construction and configuration : -----------------------------------------

  public InteractionPanel() {

    super("Visibility", "Select Interaction and Visibility", WiredBaseImage.getIcon("Scale%w", 16), WiredPlugin.IH.TRANSLATE.get());
    setLayout(new TableLayout());

    _interaction = new JPanel();
    _interaction.setLayout(new TableLayout());
    _interaction.setBorder(new TitledBorder(new EtchedBorder(), "View"));
    add(_interaction, "0 * wlt");
    _treePanel = WiredPlugin.getPlugin().getTreePanel();
    add(_treePanel, "0 * whlt");

    _group = new ButtonGroup();
    _deselect = new JToggleButton("deselect");
    _group.add(_deselect);
    _buttonsByName = new HashMap();
    _buttonsByHandler = new LinkedHashMap();
  }

  /** Adds a button for selecting the specified <tt>InteractionHandler</tt> to this panel. */
  public void addInteraction(InteractionHandler h, boolean selected) {
    if (h != null) {
      _interaction.add(makeButton(h, selected), "* * lt");
    } else {
      _interaction.add(new JLabel(), "* * ltW");
    }
  }

// -- Operations : -------------------------------------------------------------

  /**
   * Associates the specified <tt>InteractionHandler</tt> with this panel.
   * If the supplied <tt>InteractionHandler</tt> is one those that have been previously
   * added through a call to {@link #addInteraction} method, the corresponding button is
   * selected; if not, all buttons are deselected.
   */
  public void setInteractionHandler(InteractionHandler h) {
    super.setInteractionHandler(h);
    if (h == null) {
      _deselect.setSelected(true);
    } else {
      JToggleButton b = (JToggleButton) _buttonsByHandler.get(h);
      if (b != null && b.isEnabled()) {
        b.setSelected(true);
        _defaultInteractionHandler = h;
      } else {
        _deselect.setSelected(true);
      }
    }
  }

  /** Called when this panel is selected or deselected. */
  public void setSelected(boolean selected) {
    super.setSelected(selected);
    _treePanel.setSelected(selected);

    if (!selected) return;
    PageContext context = WiredPlugin.getPlugin().getCurrentPageContext();
    if (context == null) return;

    RecordPlot plot = ((WiredPage)context.getPage()).getRecordPlot();
    for (Map.Entry<InteractionHandler,JToggleButton> entry : _buttonsByHandler.entrySet()) {
      entry.getValue().setEnabled(plot.supports(entry.getKey()));
    }
  }

// -- Local methods : ----------------------------------------------------------

  private JToggleButton makeButton(final InteractionHandler h, boolean selected) {
    final JToggleButton b = new JToggleButton(h.getIcon(16), selected);
    b.setToolTipText(h.getDescription());
    b.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        WiredPlugin.getPlugin().setInteractionHandler(h);
      }
    });
    _group.add(b);
    _buttonsByName.put(h.getName(), b);
    _buttonsByHandler.put(h, b);
    return b;
  }
}
