package hep.wired.heprep.interaction;

import java.util.*;

import hep.wired.heprep.util.Keys;
import hep.wired.util.SortableListTableModel;
import org.freehep.application.Application;
import org.freehep.application.PropertyUtilities;

/**
 * TableModel for HepRep Layers.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: LayerTableModel.java 13822 2011-03-12 22:19:14Z onoprien $
 */
public class LayerTableModel extends SortableListTableModel {

// -- Private parts : ----------------------------------------------------------

  private boolean _numbersOnly;

// -- Construction : -----------------------------------------------------------

  public LayerTableModel(List<Flag> layers) {
    super(new String[]{"Layer", "Pickable"}, new Class[]{String.class, Boolean.class}, layers);
    checkForNumbersOnly();
  }
  
// -- Getters : ----------------------------------------------------------------
  
  public List<Flag> getLayers() {return keys;}

// -- Hook for action on value change : ----------------------------------------

  protected void changed() {
  }
  
// -- Implementing TableModel : ------------------------------------------------

  public boolean isCellEditable(int row, int col) {
    return (col != Flag.NAME);
  }

  public Object getValueAt(int row, int col) {
    Flag layer = (Flag) keys.get(row);
    switch (col) {
      case Flag.NAME:
        return layer.getName();
      case Flag.FLAG:
        return (Boolean) layer.isSet();
      default:
        return "";
    }
  }

  public void setValueAt(Object value, int row, int col) {
    switch (col) {
      case Flag.NAME:
        return;
      case Flag.FLAG:
        Properties userProperties = Application.getApplication().getUserProperties();
        Set notShown = new HashSet(PropertyUtilities.getStringCollection(userProperties, Keys.pickingLayer, Collections.EMPTY_LIST));

        Flag layer = (Flag) keys.get(row);
        if (((Boolean) value).booleanValue()) {
          layer.set(true);
          notShown.remove(layer.getName());
        } else {
          layer.set(false);
          notShown.add(layer.getName());
        }
        PropertyUtilities.setStringCollection(userProperties, Keys.pickingLayer, notShown);
        changed();
        return;
      default:
        return;
    }
  }

  protected int compareKeys(Object key1, Object key2, int col, boolean ascending) {
    Flag f1 = (Flag) key1;
    Flag f2 = (Flag) key2;
    switch (col) {
      default:
      case Flag.NAME:
        if (_numbersOnly) {
          return (ascending ? 1 : -1)
                  * (Double.valueOf(f1.getName()).compareTo(Double.valueOf(f2.getName())));
        } else {
          return (ascending ? 1 : -1)
                  * (f1.getName().compareTo(f2.getName()));
        }
      case Flag.FLAG:
        return (ascending ? 1 : -1)
                * (f1.isSet() ? (f2.isSet() ? 0 : 1) : f2.isSet() ? -1 : 0);
    }
  }

  public void fireTableStructureChanged() {
    checkForNumbersOnly();
    super.fireTableStructureChanged();
  }

// -- Local methods : ----------------------------------------------------------

  private void checkForNumbersOnly() {
    _numbersOnly = true;
    for (Object key : keys) {
      Flag f = (Flag) key;
      try {
        Double.parseDouble(f.getName());
      } catch (NumberFormatException e) {
        _numbersOnly = false;
        return;
      }
    }
  }

// -----------------------------------------------------------------------------
}
