// Copyright 2004-2011, FreeHEP.
package hep.wired.heprep.interaction;

import hep.graphics.heprep.HepRepInstance;
import hep.wired.services.InteractionHandler;
import hep.wired.services.RecordPlot;

import java.util.Set;

/**
 * Interface to be implemented by object picking tools.
 * Picking tools are also expected to implement {@link InteractionHandler}.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: PickHandler.java 14746 2013-05-02 18:59:56Z onoprien $
 */
public interface PickHandler {

  /**
   * Performs picking action based on current cursor location if possible,
   * decorates the plot and informs it of newly picked objects.
   */
  public void update(RecordPlot plot);

  /** Informs the plot of newly picked objects. */
  public void setSelection(RecordPlot plot, Set<HepRepInstance> selectedInstances);

  public boolean isRegionZoomable();

  public void zoomIntoRegion(RecordPlot plot);

  public boolean isPickedTranslateable(RecordPlot plot);

  public void translateToPicked(RecordPlot plot);

  public boolean canSetPickWhileDragging();

  public void setPickWhileDragging(boolean state);

  public boolean isPickWhileDragging();
}
