// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.projection;

import hep.wired.heprep.services.Projection;
import hep.wired.util.Matrix3D;

import org.freehep.xml.io.XMLIOManager;

/**
 * 3D perspective projection of (X,Y,Z) into (U,V,W).
 * 
 * @author Mark Donszelmann
 * @version $Id: PerspectiveProjection.java 8598 2006-08-14 20:38:02Z duns $
 */

public abstract class PerspectiveProjection extends ParallelProjection {

    public static class XY extends PerspectiveProjection {
        public XY() {
            super();
        }

        protected XY(Matrix3D matrix, Matrix3D resetMatrix) {
            super(matrix, resetMatrix);
        }

        public Projection copy() {
            return new XY(matrix, resetMatrix);
        }
    }

    public static class ZX extends PerspectiveProjection {
        public ZX() {
            super(new Matrix3D(0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0));
        }

        protected ZX(Matrix3D matrix, Matrix3D resetMatrix) {
            super(matrix, resetMatrix);
        }

        public Projection copy() {
            return new ZX(matrix, resetMatrix);
        }
    }

    public static class ZY extends PerspectiveProjection {
        public ZY() {
            super(new Matrix3D(0, 0, -1, 0, 1, 0, 1, 0, 0, 0, 0, 0));
        }

        protected ZY(Matrix3D matrix, Matrix3D resetMatrix) {
            super(matrix, resetMatrix);
        }

        public Projection copy() {
            return new ZY(matrix, resetMatrix);
        }
    }

    public static class XZ extends PerspectiveProjection {
        public XZ() {
            super(new Matrix3D(1, 0, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0));
        }

        protected XZ(Matrix3D matrix, Matrix3D resetMatrix) {
            super(matrix, resetMatrix);
        }

        public Projection copy() {
            return new XZ(matrix, resetMatrix);
        }
    }

    public static class YZ extends PerspectiveProjection {
        public YZ() {
            super(new Matrix3D(0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0));
        }

        protected YZ(Matrix3D matrix, Matrix3D resetMatrix) {
            super(matrix, resetMatrix);
        }

        public Projection copy() {
            return new YZ(matrix, resetMatrix);
        }
    }

    /**
     * Creates a 3D perpective projection.
     */
    protected PerspectiveProjection() {
        this(new Matrix3D());
    }

    protected PerspectiveProjection(Matrix3D matrix) {
        super(matrix);
    }

    /**
     * Used for cloning
     */
    protected PerspectiveProjection(Matrix3D matrix, Matrix3D resetMatrix) {
        super(matrix, resetMatrix);
    }

    public String getFormula() {
        return "None";
//   "(u,v,w) = " + "m00*x + m01*y + m02*z + m03, "
//                + "m10*x + m11*y + m12*z + m13, "
//                + "m20*x + m21*y + m22*z + m23";
    }

    public double[] transform(double[] xyz) {
        return super.transform(xyz);
    }

    public double[][] transform(double[][] xyz, int n) {
        return super.transform(xyz, n);
    }

    public double[] deltaTransform(double[] xyz) {
        return super.transform(xyz);
    }

    public double[] inverseTransform(double[] uvw)
            throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }

    public double[] inverseDeltaTransform(double[] uvw)
            throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }

    public String toString() {
        return super.toString() + " : \n" + matrix.toString();
    }

    // Resetable
    public Object reset(Object newState) {
        return super.reset(newState);
    }

    //
    // XMLIO
    //
    public void save(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        super.save(xmlioManager, nodeEl);
        // FIXME
    }

    public void restore(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        super.restore(xmlioManager, nodeEl);
        // FIXME
    }
}
