// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepPoint;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.heprep.util.PolyPoint;
import hep.wired.services.ViewPort;

import java.util.Iterator;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws one or more circles, centered on HepRepPoints with given "Radius",
 * positioned using "Phi" and "Theta".
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsCircle.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsCircle extends AbstractDrawAs {

    private double uvw[][] = new double[3][PolyPoint.ELLIPSEPOINTS];

    public String getKey() {
        return "circle";
    }

    public String getName() {
        return "Circle";
    }

    public String getDescription() {
        return "Draws a HepRepInstance as a Circle.";
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        Iterator iterator = instance.getPoints().iterator();
        while (iterator.hasNext()) {
            HepRepPoint center = (HepRepPoint) iterator.next();

            int n = PolyPoint.circle(center.getAttValue("radius").getDouble(),
                    center.getX(), center.getY(), center.getZ(), center
                            .getAttValue("phi").getDouble(), center
                            .getAttValue("theta").getDouble(), uvw);
            if (n == 0)
                continue;

            uvw = projection.transform(viewPort, uvw, n);

            if (atts.isFilled()) {
                graphics.setColor(atts.getFillColor());
                graphics.fillPolygon(uvw[U], uvw[V], n);
                graphics.setColor(atts.getColor());
            }
            graphics.drawPolygon(uvw[U], uvw[V], n);
        }
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());

        Iterator iterator = instance.getPoints().iterator();
        while (iterator.hasNext()) {
            HepRepPoint center = (HepRepPoint) iterator.next();
            int n = PolyPoint.circle(center.getAttValue("radius").getDouble(),
                    center.getX(), center.getY(), center.getZ(), center
                            .getAttValue("phi").getDouble(), center
                            .getAttValue("theta").getDouble(), uvw);
            if (n == 0)
                continue;
            uvw = projection.transform(viewPort, uvw, n);

            graphics.drawPolygon(uvw[U], uvw[V], n);
        }

        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());
    }
}
