// Copyright 2004-2005, FreeHEP.
package hep.wired.heprep.representation;

import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepPoint;
import hep.wired.heprep.services.Attributes;
import hep.wired.heprep.services.GraphicsMode;
import hep.wired.heprep.services.Projection;
import hep.wired.heprep.util.PolyPoint;
import hep.wired.services.ViewPort;

import java.util.Iterator;

import org.freehep.graphics2d.VectorGraphics;

/**
 * Draws one or more ellipses, centered at HepRepPoints, with given "Radius" and
 * "Radius2", positioned using "Phi", "Theta" and "Omega".
 * 
 * @author Mark Donszelmann
 * @version $Id: DrawAsEllipse.java 8598 2006-08-14 20:38:02Z duns $
 */

public class DrawAsEllipse extends AbstractDrawAs {

    private double uvw[][] = new double[3][PolyPoint.ELLIPSEPOINTS];

    public String getKey() {
        return "ellipse";
    }

    public String getName() {
        return "Ellipse";
    }

    public String getDescription() {
        return "Draws a HepRepInstance as an Ellipse.";
    }

    public void draw(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        Iterator iterator = instance.getPoints().iterator();
        while (iterator.hasNext()) {
            HepRepPoint center = (HepRepPoint) iterator.next();

            int n = PolyPoint.ellipse(center.getAttValue("radius").getDouble(),
                    center.getAttValue("radius2").getDouble(), center.getX(),
                    center.getY(), center.getZ(), center.getAttValue("phi")
                            .getDouble(), center.getAttValue("theta")
                            .getDouble(), center.getAttValue("omega")
                            .getDouble(), uvw);
            if (n == 0)
                continue;

            uvw = projection.transform(viewPort, uvw, n);

            if (atts.isFilled()) {
                graphics.setColor(atts.getFillColor());
                graphics.fillPolygon(uvw[U], uvw[V], n);
                graphics.setColor(atts.getColor());
            }
            graphics.drawPolygon(uvw[U], uvw[V], n);
        }
    }

    public void frame(VectorGraphics graphics, HepRepInstance instance,
            Attributes atts, GraphicsMode mode, Projection projection,
            ViewPort viewPort) {

        graphics.setColor(atts.getFrameColor());
        graphics.setLineWidth(atts.getFrameWidth() * 2 + atts.getLineWidth());

        Iterator iterator = instance.getPoints().iterator();
        while (iterator.hasNext()) {
            HepRepPoint center = (HepRepPoint) iterator.next();
            int n = PolyPoint.ellipse(center.getAttValue("radius").getDouble(),
                    center.getAttValue("radius2").getDouble(), center.getX(),
                    center.getY(), center.getZ(), center.getAttValue("phi")
                            .getDouble(), center.getAttValue("theta")
                            .getDouble(), center.getAttValue("omega")
                            .getDouble(), uvw);
            if (n == 0)
                continue;
            uvw = projection.transform(viewPort, uvw, n);

            graphics.drawPolygon(uvw[U], uvw[V], n);
        }

        graphics.setLineWidth(atts.getLineWidth());
        graphics.setColor(atts.getColor());
    }
}
