// Copyright 2004-2006, FreeHEP.
package hep.wired.heprep.services;

import hep.wired.services.Feature;
import hep.wired.services.ViewPort;
import hep.wired.util.UVWindices;
import hep.wired.util.WiredRegistry;
import hep.wired.util.XYZindices;

import org.freehep.xml.io.XMLIO;

/**
 * Transforms coordinates in one system (X, Y, Z) into coordinates of another
 * system (X', Y', Z'). Projections can be cascaded (sequenced).
 * 
 * @author Mark Donszelmann
 * @version $Id: Projection.java 8598 2006-08-14 20:38:02Z duns $
 */

public interface Projection extends ProjectionFeature, WiredRegistry.ID, XYZindices,
        UVWindices, XMLIO {

    /**
     * Returns the name of the projection.
     */
    public String getName();

    /**
     * Returns the formula in text readable form
     */
    public String getFormula();

    /**
     * Return true if this (composite) projection implements this featureClass.
     */
    public boolean supports(Class featureClass);

    /**
     * Return Feature if implemented, otherwise null.
     */
    public Feature getFeature(Class featureClass);

    /**
     * Sets the associated viewport.
     */
    public void setViewPort(ViewPort viewPort);

    /**
     * Returns the associated viewport.
     */
    public ViewPort getViewPort();

    /**
     * Returns an array of length 3: uvw = viewPort(projection(xyz)). The
     * returned array may have a lifetime up to the next call to this method on
     * this object. CoordinateProjection may be null.
     */
    public double[] transform(ViewPort viewPort, double[] xyz);

    /**
     * Returns an array of length 3: uvw = projection(xyz). The returned array
     * may have a lifetime up to the next call to this method on this object.
     */
    public double[] transform(double[] xyz);

    /**
     * Returns an array of length 3*n: uvw[][n] =
     * viewPort(projection(xyz[][n])). The returned array may have a lifetime up
     * to the next call to this method on this object. The number of "valid"
     * coordinates is given by n. CoordinateProjection may be null.
     */
    public double[][] transform(ViewPort viewPort, double[][] xyz, int n);

    /**
     * Returns an array of length 3*n: uvw[][n] = projection(xyz[][n]). The
     * returned array may have a lifetime up to the next call to this method on
     * this object. The number of "valid" coordinates is given by n.
     */
    public double[][] transform(double[][] xyz, int n);

    /**
     * Returns an array of length 3: uvw = delta projection(xyz). The returned
     * array may have a lifetime up to the next call to this method on this
     * object.
     */
    public double[] deltaTransform(double[] xyz);

    /**
     * Returns an array of length 3: xyz = inverse projection(uvw). The returned
     * array may have a lifetime up to the next call to this method on this
     * object. UnsupportedOperation is thrown in no inverse projection is
     * possible.
     */
// MOVED to InverseProjection
//    public double[] inverseTransform(double[] uvw)
//            throws UnsupportedOperationException;

    /**
     * Returns an array of length 3: xyz = inverse delta projection(uvw). The
     * returned array may have a lifetime up to the next call to this method on
     * this object. UnsupportedOperation is thrown in no inverse delta
     * projection is possible.
     */
// MOVED to InverseProjection
//    public double[] inverseDeltaTransform(double[] uvw)
//            throws UnsupportedOperationException;

    /**
     * Returns a deep copy of the projection.
     */
    public Projection copy();
}
