/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: HepRepTree.java 13852 2011-07-12 00:40:57Z onoprien $
 */

package hep.wired.heprep.tree;

import hep.graphics.heprep.HepRep;
import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepInstanceTree;
import hep.graphics.heprep.HepRepType;
import hep.graphics.heprep.HepRepTypeTree;
import hep.wired.util.SwingEventMulticaster;
import hep.wired.util.tree.BranchCheckBox;
import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.event.ChangeListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import hep.wired.util.tree.WiredTree;
import hep.wired.util.tree.WiredTreeModel;
import hep.wired.util.tree.WiredTreeModel.TreeVisitor;
import hep.wired.util.tree.WiredTreeNode;
import java.util.Enumeration;
import javax.swing.event.ChangeEvent;
import org.freehep.swing.popup.HasPopupItems;

public class HepRepTree extends WiredTree implements HasPopupItems {

// -- Private parts : ----------------------------------------------------------

  private ChangeListener _listener;

// -- Construction : -----------------------------------------------------------

  public HepRepTree() {
    super((WiredTreeModel)null);
  }

  public HepRepTree(WiredTreeModel model) {
    super(model);
  }

// -- Setters : ----------------------------------------------------------------

  public void setModel(TreeModel newModel) {
    super.setModel(newModel);
  }

// -- Popup menues : -----------------------------------------------------------

  public JPopupMenu modifyPopupMenu(JPopupMenu menu, Component source, Point p) {

    final TreePath path = getClosestPathForLocation(p.x, p.y);
    Rectangle r = getPathBounds(path);
    menu.removeAll();
    if (r == null || ! r.contains(p)) return menu;

    menu.add(new JMenuItem("Expand all")).addActionListener(
            new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                TreeVisitor v = new TreeVisitor() {
                  public boolean visit(TreePath tp) {
                    if (((WiredTreeNode) tp.getLastPathComponent()).isLeaf()) {
                      return false;
                    } else {
                      HepRepTree.this.expandPath(tp);
                      return true;
                    }
                  }
                };
                v.visitPaths(path);
              }
            });

    menu.add(new JMenuItem("Collapse all")).addActionListener(
            new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                Enumeration<WiredTreeNode> ee = ((WiredTreeNode)path.getLastPathComponent()).postorderEnumeration();
                while (ee.hasMoreElements()) {
                  WiredTreeNode node = ee.nextElement();
                  TreePath path = new TreePath(node.getPath());
                  if (!node.isLeaf()) collapsePath(path);
                }
              }
            });

    menu.addSeparator();

    menu.add(new JMenuItem("Show all")).addActionListener(
            new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                final WiredTreeNode node = (WiredTreeNode) path.getLastPathComponent();
                TreeVisitor v = new TreeVisitor() {
                  public boolean visit(WiredTreeNode n) {
                    n.setMarkedNode(true);
                    if (n.getBranchState() == BranchCheckBox.State.ALL) {
                      return false;
                    } else {
                      n.setMarkedBranch(true);
                      return true;
                    }
                  }
                };
                v.visitNodes(node);
                HepRepTree.this.repaint();
                getModel().fireMarkChanged();
              }
            });

    menu.add(new JMenuItem("Hide all")).addActionListener(
            new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                final WiredTreeNode node = (WiredTreeNode) path.getLastPathComponent();
                TreeVisitor v = new TreeVisitor() {
                  public boolean visit(WiredTreeNode n) {
                    n.setMarkedNode(false);
                    return true;
                  }
                };
                v.visitNodes(node);
                HepRepTree.this.repaint();
                getModel().fireMarkChanged();
              }
            });

    menu.addSeparator();


    menu.add(new JMenuItem("Invert all")).addActionListener(
            new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                final WiredTreeNode node = (WiredTreeNode) path.getLastPathComponent();
                TreeVisitor v = new TreeVisitor() {
                  public boolean visit(WiredTreeNode n) {
                    n.setMarkedNode(!n.isMarkedNode());
                    n.setMarkedBranch(true);
                    return true;
                  }
                };
                v.visitNodes(node);
                HepRepTree.this.repaint();
              }
            });

    return menu;
  }

// -- Customize display : ------------------------------------------------------

  public String convertValueToText(Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
    return HepRepTreeModel.getName(value);
  }

// -- Handling change listeners (for compatibility with Mark's code) : ---------

  public void addChangeListener(ChangeListener a) {
    _listener = SwingEventMulticaster.add(_listener, a);
  }

  public void removeChangeListener(ChangeListener r) {
    _listener = SwingEventMulticaster.remove(_listener, r);
  }

  private void fireStateChanged() {
    if (_listener != null) {
      _listener.stateChanged(new ChangeEvent(this));
    }
  }

// -----------------------------------------------------------------------------

}
