// Copyright 2004-2006, FreeHEP
package hep.wired.heprep.util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.TexturePaint;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.RenderedImage;
import java.io.IOException;
import java.text.AttributedCharacterIterator;

import org.freehep.graphics2d.TagString;
import org.freehep.graphics2d.VectorGraphics;
import org.freehep.graphicsio.AbstractVectorGraphicsIO;

/**
 * Calculates the bounding box as objects are drawn into this Graphics2D
 * context.
 * 
 * FIXME WIRED-394, fairly incomplete! FIXME WIRED-257 should maybe move to
 * FreeHEP...
 * 
 * @author Mark Donszelmann
 * @version $Id: BoundingBoxGraphics2D.java 8598 2006-08-14 20:38:02Z duns $
 */
public class BoundingBoxGraphics2D extends AbstractVectorGraphicsIO {
    /*
     * ================================================================================
     * Table of Contents: ------------------ 1. Constructors & Factory Methods
     * 2. Document Settings 3. Header, Trailer, Multipage & Comments 3.1 Header &
     * Trailer 3.2 MultipageDocument methods 4. Create & Dispose 5. Drawing
     * Methods 5.1. shapes (draw/fill) 5.1.1. lines, rectangles, round
     * rectangles 5.1.2. polylines, polygons 5.1.3. ovals, arcs 5.1.4. shapes
     * 5.2. Images 5.3. Strings 6. Transformations 7. Clipping 8. Graphics State /
     * Settings 8.1. stroke/linewidth 8.2. paint/color 8.3. font 8.4. rendering
     * hints 9. Auxiliary 10. Private/Utility Methos
     * ================================================================================
     */

    private Rectangle2D boundingBox = null;

    private boolean excludeStrings = false;

    // private AffineTransform transform = new AffineTransform();

    /**
     * Returns the current bounding box.
     */
    public Rectangle2D getBoundingBox() {
        return boundingBox;
    }

    private void union(Shape shape) {
        Rectangle2D bounds = shape.getBounds2D();

        // ignore invalid shapes
        if ((bounds.getWidth() <= 0) && (bounds.getHeight() <= 0))
            return;

        boundingBox = (boundingBox == null) ? bounds : bounds
                .createUnion(boundingBox);
    }

    /*
     * ================================================================================
     * 1. Constructors & Factory Methods
     * ================================================================================
     */
    /**
     * Create a bounding box graphics context with dimension 800x600.
     */
    public BoundingBoxGraphics2D() {
        this(false);
    }

    public BoundingBoxGraphics2D(boolean excludeStrings) {
        super(new Dimension(800, 600), false);
        this.excludeStrings = excludeStrings;
        // Create a graphics context with given imageBounds.
        // This constructor is used by the user to create the initial graphics
        // context.
        // doRestoreOnDispose is used to call writeGraphicsRestore(),
        // when the graphics context is being disposed off.
    }

    /**
     * Create a sub context.
     */
    protected BoundingBoxGraphics2D(BoundingBoxGraphics2D graphics,
            boolean doRestoreOnDispose) {
        super(graphics, doRestoreOnDispose);
        System.out.println(getClass() + ": Cloned");
        // Create a graphics context from a given graphics context.
        // This constructor is used by the system to clone a given graphics
        // context.
        // doRestoreOnDispose is used to call writeGraphicsRestore(),
        // when the graphics context is being disposed off.
        boundingBox = graphics.boundingBox;
        excludeStrings = graphics.excludeStrings;
        // transform = graphics.transform;
    }

    /*
     * ================================================================================ |
     * 2. Document Settings
     * ================================================================================
     */

    /*
     * ================================================================================ |
     * 3. Header, Trailer, Multipage & Comments
     * ================================================================================
     */
    /* 3.1 Header & Trailer */
    public void writeHeader() throws IOException {
        writeWarning(getClass() + ": writeHeader() not implemented.");
        // Write out the header to the output stream.
    }

    public void writeBackground() throws IOException {
        writeWarning(getClass() + ": writeBackground() not implemented.");
        // Write out the background to the output stream.
    }

    public void writeTrailer() throws IOException {
        writeWarning(getClass() + ": writeTrailer() not implemented.");
        // Write out the trailer to the output stream.
    }

    public void closeStream() throws IOException {
        writeWarning(getClass() + ": closeStream() not implemented.");
        // Close the output stream.
    }

    /* 3.2 MultipageDocument methods */

    /*
     * ================================================================================
     * 4. Create & Dispose
     * ================================================================================
     */

    public Graphics create() {
        // Create a new graphics context from the current one.
        try {
            // Save the current context for restore later.
            writeGraphicsSave();
        } catch (IOException e) {
        }
        // The correct graphics context should be created.
        return new BoundingBoxGraphics2D(this, false);
    }

    public Graphics create(double x, double y, double width, double height) {
        // Create a new graphics context from the current one.
        try {
            // Save the current context for restore later.
            writeGraphicsSave();
        } catch (IOException e) {
        }
        // The correct graphics context should be created.
        VectorGraphics graphics = new BoundingBoxGraphics2D(this, false);
        graphics.clipRect(x, y, width, height);
        return graphics;
    }

    protected void writeGraphicsSave() throws IOException {
        writeWarning(getClass() + ": writeGraphicsSave() not implemented.");
        // Write a graphics context save.
        // If the output format does not support this, keep a stack yourself.
    }

    protected void writeGraphicsRestore() throws IOException {
        writeWarning(getClass() + ": writeGraphicsRestore() not implemented.");
        // Write a graphics context restore.
        // If the output format does not support this, keep a stack yourself.
    }

    /*
     * ================================================================================ |
     * 5. Drawing Methods
     * ================================================================================
     */
    /* 5.1.4. shapes */

    public void draw(Shape shape) {
        union(shape);
    }

    public void fill(Shape shape) {
        union(shape);
    }

    public void fillAndDraw(Shape shape, Color fillColor) {
        union(shape);
    }

    /* 5.2. Images */
    public void copyArea(int x, int y, int width, int height, int dx, int dy) {
        writeWarning(getClass()
                + ": copyArea(int, int, int, int, int, int) not implemented.");
        // Mostly unimplemented.
    }

    protected void writeImage(RenderedImage image, AffineTransform xform,
            Color bkg) throws IOException {
        writeWarning(getClass()
                + ": writeImage(RenderedImage, AffineTransform, Color) not implemented.");
        // Write out the image.
    }

    /* 5.3. Strings */
    protected void writeString(String string, double x, double y)
            throws IOException {
        writeWarning(getClass()
                + ": drawString(String, double, double) not implemented.");
        // Write out the string.
    }

    public void drawString(AttributedCharacterIterator iterator, float x,
            float y) {
        writeWarning(getClass()
                + ": drawString(AttributedCharacterIterator, float, float) not implemented.");
        // Write out the string.
    }

    public void drawGlyphVector(GlyphVector g, float x, float y) {
        writeWarning(getClass()
                + ": drawGlyphVector(GlyphVector, float, float) not implemented.");
        // Write out the string.
    }

    public void drawString(TagString str, double x, double y) {
        if (!excludeStrings)
            super.drawString(str, x, y);
    }

    public void drawString(String str, double x, double y, int horizontal,
            int vertical) {
        if (!excludeStrings)
            super.drawString(str, x, y, horizontal, vertical);
    }

    public void drawString(TagString str, double x, double y, int horizontal,
            int vertical) {
        if (!excludeStrings)
            super.drawString(str, x, y, horizontal, vertical);
    }

    public void drawString(String str, double x, double y, int horizontal,
            int vertical, boolean framed, Color frameColor, double frameWidth,
            boolean banner, Color bannerColor) {
        if (!excludeStrings)
            super.drawString(str, x, y, horizontal, vertical, framed,
                    frameColor, frameWidth, banner, bannerColor);
    }

    public void drawString(TagString str, double x, double y, int horizontal,
            int vertical, boolean framed, Color frameColor, double frameWidth,
            boolean banner, Color bannerColor) {
        if (!excludeStrings)
            super.drawString(str, x, y, horizontal, vertical, framed,
                    frameColor, frameWidth, banner, bannerColor);
    }

    /*
     * ================================================================================ |
     * 6. Transformations
     * ================================================================================
     */
    protected void writeTransform(AffineTransform t) throws IOException {
        writeWarning(getClass()
                + ": writeTransform(AffineTransform) not implemented.");
        // Write out the transform to be applied to the internal transform of
        // the output
        // format.
        // You can also use the currentTransform.
    }

    protected void writeSetTransform(AffineTransform t) throws IOException {
        writeWarning(getClass()
                + ": writeSetTransform(AffineTransform) not implemented.");
    }
    
    /*
     * ================================================================================ |
     * 7. Clipping
     * ================================================================================
     */
    // FVG-9 FIXME, remove after JAS upgrades to latest VG
    protected void writeClip(Rectangle2D r2d) throws IOException {
        writeWarning(getClass() + ": writeClip(Rectangle2D) not implemented.");
        // Write out the clip rectangle.
    }

    protected void writeClip(Shape s) throws IOException {
        writeWarning(getClass() + ": writeClip(Shape) not implemented.");
        // Write out the clip shape.
    }

    protected void writeSetClip(Shape s) throws IOException {
        writeWarning(getClass() + ": writeSetClip(Shape) not implemented.");
        // Write out the clip shape.
    }
    /*
     * ================================================================================ |
     * 8. Graphics State
     * ================================================================================
     */
    /* 8.1. stroke/linewidth */
    protected void writeWidth(float width) throws IOException {
    }

    protected void writeCap(int cap) throws IOException {
    }

    protected void writeJoin(int join) throws IOException {
    }

    protected void writeMiterLimit(float limit) throws IOException {
    }

    protected void writeDash(float[] dash, float phase) throws IOException {
    }

    /* 8.2. paint/color */
    public void setPaintMode() {
    }

    public void setXORMode(Color c1) {
    }

    protected void writePaint(Color p) throws IOException {
    }

    protected void writePaint(GradientPaint p) throws IOException {
    }

    protected void writePaint(TexturePaint p) throws IOException {
    }

    protected void writePaint(Paint p) throws IOException {
    }

    /* 8.3. font */
    protected void writeFont(Font font) throws IOException {
    }
    /* 8.4. rendering hints */

    /*
     * ================================================================================ |
     * 9. Auxiliary
     * ================================================================================
     */
    public GraphicsConfiguration getDeviceConfiguration() {
        writeWarning(getClass() + ": getDeviceConfiguration() not implemented.");
        // Mostly unimplemented
        return null;
    }

    public boolean hit(Rectangle rect, Shape s, boolean onStroke) {
        writeWarning(getClass()
                + ": hit(Rectangle, Shape, boolean) not implemented.");
        // Mostly unimplemented
        return false;
    }

    public void writeComment(String comment) throws IOException {
        writeWarning(getClass() + ": writeComment(String) not implemented.");
        // Write out the comment.
    }

    public void writeWarning(String warning) {
        System.err.println(warning);
    }

    public String toString() {
        return "BoundingBoxGraphics2D: " + boundingBox;
    }
}
