// Copyright 2004, FreeHEP.
package hep.wired.heprep.util;

import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;

import org.freehep.graphicsio.CubicToLinePathConstructor;

/**
 * Constructs a path while keeping track if the path crosses or is within a
 * given rectangle.
 * 
 * @author Mark Donszelmann
 * @version $Id: InsideRectanglePathConstructor.java 289 2004-06-22 22:59:36Z
 *          duns $
 */
public class InsideRectanglePathConstructor extends CubicToLinePathConstructor {

    private Rectangle2D rectangle;

    private transient Line2D line = new Line2D.Double();

    /**
     * Creates a path constructor with given rectangle
     */
    public InsideRectanglePathConstructor(Rectangle2D rectangle) {
        this.rectangle = rectangle;
    }

    /**
     * Adds a line to the path, called from addPath.
     */
    public void line(double x, double y) throws IOException {
        line.setLine(currentX, currentY, x, y);
        if (rectangle.intersectsLine(line)) {
            lineFound(new Line2D.Double(currentX, currentY, x, y));
        }
        super.line(x, y);
    }

    /**
     * Closes the current path, called from addPath.
     */
    public void closePath(double x0, double y0) throws IOException {
        line(x0, y0);
        super.closePath(x0, y0);
    }

    /**
     * Called when a line is found that is inside the rectangle or crosses it,
     * so that the subclass can use this to track some other information, such
     * as keeping a reference of the object(s).
     */
    protected void lineFound(Line2D line) {
    }
}
