// Copyright 2005-2011, FreeHEP.
package hep.wired.heprep.util;

import hep.graphics.heprep.HepRep;
import hep.graphics.heprep.HepRepAttDef;
import hep.graphics.heprep.HepRepDefaults;
import hep.graphics.heprep.HepRepInstance;
import hep.graphics.heprep.HepRepPoint;
import hep.graphics.heprep.HepRepType;
import hep.graphics.heprep.HepRepTypeTree;
import hep.wired.util.XYZindices;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * FIXME WIRED-395 most or all of these methods should maybe move to jheprep.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredHepRepUtil.java 13825 2011-05-04 18:31:00Z onoprien $
 */
public class WiredHepRepUtil implements XYZindices {

    private WiredHepRepUtil() {
    }

    public static Set/* <String> */getCategories(HepRep heprep) {
        Set categories = new HashSet();
        // defaults categories
        categories.addAll(getCategories(HepRepDefaults.getAttDefs()));

        // other categories
        for (Iterator i = heprep.getTypeTreeList().iterator(); i.hasNext();) {
            HepRepTypeTree typeTree = (HepRepTypeTree) i.next();
            categories.addAll(getCategories(typeTree.getTypeList()));
        }
        return categories;
    }

    public static Set/* <String> */getCategories(List/* <HepRepType> */types) {
        Set categories = new HashSet();
        for (Iterator i = types.iterator(); i.hasNext();) {
            HepRepType type = (HepRepType) i.next();
            categories.addAll(getCategories(type.getAttDefsFromNode()));
            categories.addAll(getCategories(type.getTypeList()));
        }
        return categories;
    }

    public static Set/* <String> */getCategories(
            Set/* <HepRepAttDef> */attDefs) {
        Set categories = new HashSet();
        for (Iterator j = attDefs.iterator(); j.hasNext();) {
            HepRepAttDef attDef = (HepRepAttDef) j.next();
            categories.add(attDef.getCategory());
        }
        return categories;
    }

    public static String getHepRepName(Object object) {
        if (object instanceof HepRepType) {
            return ((HepRepType) object).getName();
        } else if (object instanceof HepRepTypeTree) {
            return ((HepRepTypeTree) object).getName();
        } else if (object instanceof HepRep) {
            return "HepRep";
        } else {
            return object.getClass().getName();
        }
    }


    /**
     * Calculates a center point of all points within this instance (and points
     * of subinstances).
     */
    public static double[] getCenterPoint(HepRepInstance instance) {
        int n = 0;
        double[] t = null;

        List instances = instance.getInstances();
        for (Iterator j = instances.iterator(); j.hasNext();) {
            HepRepInstance i = (HepRepInstance) j.next();
            double[] c = getCenterPoint(i);
            if (c != null) {
                if (t == null) {
                    t = c;
                } else {
                    t[X] += c[X];
                    t[Y] += c[Y];
                    t[Z] += c[Z];
                }
                n++;
            }
        }

        List points = instance.getPoints();
        for (Iterator i = points.iterator(); i.hasNext();) {
            HepRepPoint p = (HepRepPoint) i.next();
            if (t == null) {
                t = new double[3];
                t[X] = p.getX();
                t[Y] = p.getY();
                t[Z] = p.getZ();
            } else {
                t[X] += p.getX();
                t[Y] += p.getY();
                t[Z] += p.getZ();
            }
            n++;
        }

        if ((n > 0) && (t != null)) {
            t[X] /= n;
            t[Y] /= n;
            t[Z] /= n;
        }
        return t;
    }

    public static Set/* <HepRepInstance> */getInstancesAndChildren(
            Collection/* <HepRepInstance> */instances) {
        Set s = new HashSet();
        s.addAll(instances);
        for (Iterator j = instances.iterator(); j.hasNext();) {
            HepRepInstance i = (HepRepInstance) j.next();
            s.addAll(getInstancesAndChildren(i.getInstances()));
        }
        return s;
    }
}
